{****************************************************}
{}
{ 	CalendarCDEF.p}
{}
{		Version 1.1	}
{}
{		93/7/31		created	}
{}
{	CDEF ID = 11 }
{	ProcID = (CDEF ID * 16) + variation code }
{	Variation code: }
{	1: draws a shadow }
{}
{		The value of the control contains the number of days since 1/1/1904.	}
{		If you want to use the International Utilites of the toolbox multiplay the value	}
{		by 24*60*60 to obtain "raw" seconds.}
{		If you specify a value of -1 the date from the internal clock will be shown.	}
{		To get the next year or month click in the year or month box on the top.	}
{		You will get the previous year or month respectively if you hold down the shift key.	}
{}
{		Author:		}
{		Hansruedi Baer		}
{		Department of Geography	}
{		University of Zurich		}
{		Winterthurerstrasse 190	}
{		CH-8057   Z u r i c h 		}
{		e-mail: baer@gis.geogr.unizh.ch	}
{}
{	History:	}
{}
{	Version 1.1	1994/2/19	}
{		 Option key instead of shift key to scroll backwards in time}
{		 Increased delay when scrolling (10 tics)}
{}
{****************************************************}

unit CalendarCDEF;

interface


	function Main (varCode: integer; control: ControlHandle; message: integer; param: longint): longint;


implementation


{****************************************************}
{	CalendarCDEF	}
{****************************************************}
	function CalendarCDEF (varCode: integer; control: ControlHandle; message: integer; param: longint): longint;
	Forward;


{****************************************************}
{	Main	}
{****************************************************}
	function Main (varCode: integer; control: ControlHandle; message: integer; param: longint): longint;
	begin
		Main := CalendarCDEF(varCode, control, message, param);
	end;		{ Main }


{****************************************************}
{	CalendarCDEF	}
{****************************************************}
	function CalendarCDEF (varCode: integer; control: ControlHandle; message: integer; param: longint): longint;

		const
			kdaysecs = 86400;		{ number of seconds of a day }
			know = -1;				{ control value used to indicate the date of the internal clock }
			hBox = 16;				{ width of box in pixels }
			vBox = 12;				{ heigth of box in pixels }
			kOption = $3A;			{ key code for option key }
			kShift = $38;			{ key code for shift key }
			kxlin = 45;				{ position of separator line }
			inCalendar = 30;		{ part code returned }
			ktics = 10;				{ delay when scrolling }
			minYear = 1904;
			maxYear = 2039;

		var
			tableRect, yearRect, monthRect, nowRect: Rect;
			secs: Longint;
			result: longint;
			savePen: PenState;
			txFont, txSize: integer;
			txFace: Style;
			finalTick: Longint;


{****************************************************}
{	DoInit	}
{****************************************************}
		procedure DoInit;
		begin
			control^^.contrlData := Handle(NewRgn);
		end; 	{ DoInit }


{****************************************************}
{	DoDispose	}
{****************************************************}
		procedure DoDispose;
		begin
			DisposeRgn(RgnHandle(control^^.contrlData));
		end; 	{ DoDispose }


{****************************************************}
{	KeyIsDown	}
{}
{		Check if key with theKeyCode is pressed.	}
{****************************************************}
		function KeyIsDown (theKeyCode: integer): Boolean;
			var
				theKeys: KeyMap;
		begin
			GetKeys(theKeys);
			KeyIsDown := theKeys[theKeyCode];
		end; 	{ KeyIsDown }


{****************************************************}
{	SetEnv	}
{}
{		Set the graphics environment.	}
{****************************************************}
		procedure SetEnv;
			var
				clipRgn: RgnHandle;
				clipRct: Rect;
		begin
			with control^^ do
				begin
					txFont := contrlOwner^.txFont;
					txSize := contrlOwner^.txSize;
					txFace := contrlOwner^.txFace;
					clipRgn := RgnHandle(contrlData);
					GetClip(clipRgn);
					SetOrigin(-contrlRect.left, -contrlRect.top - (2 * vBox + 2));
				end;
			TextFont(applFont);
			TextSize(9);
			GetClip(clipRgn);
			SetRect(clipRct, 0, -2 * vBox - 2, 7 * hBox + 2, 6 * vBox + 2);
			ClipRect(clipRct);
		end;	{ SetEnv }

{****************************************************}
{	ResetEnv	}
{}
{		Restore the old graphics environment.	}
{****************************************************}
		procedure ResetEnv;
			var
				clipRgn: RgnHandle;
		begin
			TextFont(txFont);
			TextSize(txSize);
			TextFace(txFace);
			SetOrigin(0, 0);
			clipRgn := RgnHandle(control^^.contrlData);
			SetClip(clipRgn);
			SetEmptyRgn(clipRgn);
		end;	{ ResetEnv }


{****************************************************}
{	GetDays	}
{}
{		Return the number of days since 1/1/1904.	}
{****************************************************}
		function GetDays: integer;
			var
				dtrp: DateTimeRec;
				temp: longint;
		begin
			Secs2Date(secs, dtrp);
			with dtrp do
				begin
					hour := 0;
					minute := 0;
					second := 0;
				end;
			Date2Secs(dtrp, temp);
			GetDays := temp div kdaysecs;
		end;	{ GetDays}


{****************************************************}
{	GetDay	}
{}
{		Return the day of the current month.	}
{****************************************************}
		function GetDay: integer;
			var
				dtrp: DateTimeRec;
		begin
			Secs2Date(secs, dtrp);
			GetDay := dtrp.day;
		end;	{ GetDay }


{****************************************************}
{	GetFirstDay	}
{}
{		Return the day of the week of the first day of the current month.	}
{****************************************************}
		function GetFirstDay: integer;
			var
				dtrp: DateTimeRec;
				temp: longint;
		begin
			Secs2Date(secs, dtrp);
			temp := secs - kdaysecs * (dtrp.day - 1);
			Secs2Date(temp, dtrp);
			GetFirstDay := dtrp.dayOfWeek;
		end;	{ GetFirstDay }


{****************************************************}
{	GetNrofDays	}
{}
{		Return the number of days of the current month.	}
{****************************************************}
		function GetNrofDays: integer;
			var
				dtrp: DateTimeRec;
				temp: Longint;
				month: integer;
		begin
			Secs2Date(secs, dtrp);
			month := dtrp.month;
			temp := secs + (27 - dtrp.day) * kdaysecs;
			Secs2Date(temp, dtrp);
			while (month = dtrp.month) do
				begin
					GetNrofDays := dtrp.day;
					temp := temp + kdaysecs;
					Secs2Date(temp, dtrp);
				end;
		end;	{ GetNrofDays }


{****************************************************}
{	GetDayRect	}
{}
{		Calculate the size of the rectangle this number occupies.	}
{****************************************************}
		function GetDayRect: Rect;
			var
				nthDay: integer;
				selRect: Rect;
		begin
			nthDay := GetFirstDay + GetDay - 2;		{ zero based index}
			SetRect(selRect, 1 + hBox * (nthDay mod 7), 1 + vBox * (nthDay div 7), hBox * (1 + nthDay mod 7), vBox * (1 + nthDay div 7));
			GetDayRect := selRect;
		end;	{ GetDayRect }


{****************************************************}
{	ChangeDate	}
{}
{		Add year, month, or day to th current date.	}
{****************************************************}
		procedure ChangeDate (year, month, day: integer);
			var
				dtrp: DateTimeRec;
				newMonth, newYear: integer;
		begin
			Secs2Date(secs, dtrp);
			newMonth := dtrp.month + month;
			newYear := dtrp.year + year;
			while (newMonth < 1) do
				begin
					newMonth := newMonth + 12;
					newYear := newYear - 1;
				end;
			while (newMonth > 12) do
				begin
					newMonth := newMonth - 12;
					newYear := newYear + 1;
				end;
			if (newYear < minYear) then
				newYear := minYear
			else if (newYear > maxYear) then
				newYear := maxYear
			else
				begin
					dtrp.year := newYear;
					dtrp.month := newMonth;
					dtrp.day := dtrp.day + day;
					Date2Secs(dtrp, secs);
				end;
		end;	{ ChangeDate }


{****************************************************}
{	SetNewDay	}
{}
{		Convert mouse position to number of day.	}
{****************************************************}
		procedure SetNewDay (mousePos: Point);
			var
				dtrp: DateTimeRec;
				day, index: integer;
		begin
			index := 7 * (mousePos.v div vBox) + mousePos.h div hBox;
			day := index - GetFirstDay + 2;
			if (day < 1) then
				day := 1
			else if (day > GetNrofDays) then
				day := GetNrofDays;
			ChangeDate(0, 0, day - GetDay);
		end;	{ SetNewDay }


{****************************************************}
{	DrawFrame	}
{}
{		Draw the frame of the control.	}
{		You should adjust the size of the control's rectangle to the correct value.	}
{****************************************************}
		procedure DrawFrame;
			var
				aRect: Rect;
		begin
			SetRect(aRect, 0, -2 * vBox - 2, 7 * hBox + 1, 6 * vBox + 1);
			FrameRect(aRect);
			if (varCode = 1) then
				begin
					moveTo(aRect.left + 2, aRect.bottom);
					lineTo(aRect.right, aRect.bottom);
					lineTo(aRect.right, aRect.top + 2);
				end;
		end;	{ DrawFrame }


{****************************************************}
{	DrawGrid	}
{}
{		Draws all lines in the table.	}
{****************************************************}
		procedure DrawGrid;

			var
				j, i: integer;
				xpos, ypos: integer;
		begin

			ypos := 0;
			for j := 1 to 6 do
				begin
					moveto(1, ypos);
					line(7 * hBox - 2, 0);
					ypos := ypos + vBox;
				end;

			xpos := 0;
			for i := 1 to 6 do
				begin
					xpos := xpos + hBox;
					moveto(xpos, 1);
					line(0, 6 * vBox - 2);
				end;

		end;	{ DrawGrid }


{****************************************************}
{	DrawNumbers	}
{}
{		Draw all day numbers in the table.	}
{****************************************************}
		procedure DrawNumbers;
			var
				thePat: pattern;
				dstr: Str255;
				invRect: Rect;
				day, firstDay, nDays, count: integer;
				j, i: integer;
				xpos, ypos: integer;
		begin

			firstDay := GetFirstDay;
			nDays := GetNrofDays;
			xpos := 0;
			ypos := 0;
			day := 1;
			count := 1;
			for j := 1 to 6 do
				begin
					for i := 1 to 7 do
						begin
							SetRect(invRect, 1 + (i - 1) * hBox, 1 + (j - 1) * vBox, i * hBox, j * vBox);
							EraseRect(invRect);
							NumToString(day, dstr);
							moveto(xpos + 15 - StringWidth(dstr), ypos + 10);
							if (count >= firstDay) & (day <= nDays) then
								begin
									DrawString(dstr);
									day := day + 1;
								end;
							count := count + 1;
							xpos := xpos + hBox;
						end;
					xpos := 0;
					ypos := ypos + vBox;
				end;
			invRect := GetDayRect;
			if (control^^.contrlHilite <> 255) then
				InvertRect(invRect)
			else
				begin
					StuffHex(@thePat, 'AA55AA55AA55AA55');	{ gray }
					PenPat(thePat);
					FrameRect(invRect);
				end;
		end;	{ DrawNumbers }


{****************************************************}
{	DrawDaysofWeek	}
{}
{		Draws the abbrevation of the days of the week.	}
{****************************************************}
		procedure DrawDaysofWeek;
			var
				hIntl: Intl1Hndl;
				dstr: str255;
				dtrp: DateTimeRec;
				i, xpos: integer;
		begin
			hIntl := Intl1Hndl(IUGetIntl(1));
			xpos := 0;
			moveTo(1, -1);
			line(7 * hBox - 2, 0);
			moveTo(1, -vBox - 1);
			line(7 * hBox - 2, 0);
			for i := 1 to 7 do
				begin
					moveto(6 + xpos, -3);
					dstr := hIntl^^.days[i];
					DrawChar(dstr[1]);
					xpos := xpos + hBox;
				end;
		end;	{ DrawDaysofWeek }


{****************************************************}
{	DrawMonth	}
{}
{		Draws the current month in top line.	}
{****************************************************}
		procedure DrawMonth;
			var
				hIntl: Intl1Hndl;
				dstr: str255;
				dtrp: DateTimeRec;
		begin
			EraseRect(monthRect);
			hIntl := Intl1Hndl(IUGetIntl(1));
			Secs2Date(secs, dtrp);
			dstr := hIntl^^.months[dtrp.month];
			moveto(1 + kxlin + (7 * hBox - kxlin - StringWidth(dstr)) div 2, -16);
			DrawString(dstr);
		end;	{ DrawMonth }


{****************************************************}
{	DrawYear	}
{}
{		Draws the current year in top line.	}
{****************************************************}
		procedure DrawYear;
			var
				hIntl: Intl1Hndl;
				dstr: str255;
				dtrp: DateTimeRec;
		begin
			EraseRect(yearRect);
			moveTo(kxlin, -vBox - 2);
			line(0, -vBox + 1);
			Secs2Date(secs, dtrp);
			NumToString(dtrp.year, dstr);
			moveto(1 + (kxlin - StringWidth(dstr)) div 2, -16);
			DrawString(dstr);
		end;	{ DrawYear }


{****************************************************}
{	DrawCalendar	}
{}
{		Draws all parts of the calendar.		}
{****************************************************}
		procedure DrawCalendar;
		begin
			DrawGrid;
			DrawNumbers;
			DrawFrame;
			DrawDaysofWeek;
			TextFace([bold]);
			DrawMonth;
			DrawYear;
		end;	{ DrawCalendar }


{****************************************************}
{	DoDraw	}
{}
{		Draws all parts of the calendar.		}
{****************************************************}
		procedure DoDraw;
			var
				partCode: integer;
		begin
			partCode := LoWord(param);	{high word is garbage}
			HLock(Handle(control));
			with control^^ do
				begin
					if contrlVis <> 0 then
						begin
							SetEnv;
							if partCode = 0 then
								DrawCalendar
							else if partCode = 255 then
								begin
									if contrlHilite = 255 then
										begin			{ deactive }
											DrawNumbers;
										end
									else
										begin			{ activate }
											DrawNumbers;
										end;
								end;
							ResetEnv;
						end;
				end; {with}
			HUnlock(Handle(control));
		end;	{ DoDraw }


{****************************************************}
{	Update	}
{}
{		Update the contents of the table.	}
{****************************************************}
		procedure Update;
		begin
			TextFace([bold]);
			DrawYear;
			DrawMonth;
			TextFace([]);
			DrawNumbers;
		end;	{ Update }


{****************************************************}
{	TestDay	}
{}
{		Test which day is selected.	}
{****************************************************}
		procedure TestDay;
			var
				dayRect, newDayRect, resultRect: Rect;
				mousePos: Point;
		begin
			while StillDown do
				begin
					GetMouse(mousePos);
					if PtInRect(mousePos, tableRect) then
						begin
							dayRect := GetDayRect;
							if (not PtInRect(mousePos, dayRect)) then
								begin
									SetNewDay(mousePos);
									newDayRect := GetDayRect;
									if not (SectRect(dayRect, newDayRect, resultRect)) then
										begin
											InvertRect(dayRect);
											InvertRect(newDayRect);
										end;
								end;
						end;
				end;
		end;	{ TestDay }


{****************************************************}
{	TestYear	}
{}
{		Test which year is selected.	}
{****************************************************}
		procedure TestYear;
			var
				mousePos: Point;
		begin
			while StillDown do
				begin
					GetMouse(mousePos);
					if PtInRect(mousePos, yearRect) then
						begin
							if KeyIsDown(kOption) then
								ChangeDate(-1, 0, 0)
							else
								ChangeDate(1, 0, 0);
							if not KeyIsDown(kShift) then
								Delay(ktics, finalTick);
							Update;
						end;
				end;
		end;	{ TestYear }


{****************************************************}
{	TestMonth	}
{}
{		Test which month is selected.	}
{****************************************************}
		procedure TestMonth;
			var
				mousePos: Point;
		begin
			while StillDown do
				begin
					GetMouse(mousePos);
					if PtInRect(mousePos, monthRect) then
						begin
							if KeyIsDown(kOption) then
								ChangeDate(0, -1, 0)
							else
								ChangeDate(0, 1, 0);
							if not KeyIsDown(kShift) then
								Delay(ktics, finalTick);
							Update;
						end;
				end;
		end;	{ TestMonth }


{****************************************************}
{	TestNow	}
{}
{		Check if the current date is selected.	}
{****************************************************}
		procedure TestNow;
		begin
			GetDateTime(secs);
			Update;
			while Stilldown do
				;
		end;	{ TestNow }


{****************************************************}
{	DoTest	}
{}
{		Check which item is selected.	}
{****************************************************}
		procedure DoTest;
			var
				mousePos: Point;
		begin
			with control^^ do
				begin
					if contrlHilite <> 255 then
						begin
							mousePos := Point(param);

							if PtInRect(mousePos, contrlRect) then
								begin
									result := inCalendar;
									SetEnv;
									mousePos.h := mousePos.h - contrlRect.left;
									mousePos.v := mousePos.v - contrlRect.top - (2 * vBox + 2);

									if PtInRect(mousePos, tableRect) then
										TestDay

									else if PtInRect(mousePos, yearRect) then
										TestYear

									else if PtInRect(mousePos, monthRect) then
										TestMonth

									else if PtInRect(mousePos, nowRect) & StillDown then
										TestNow;

									ResetEnv;
								end;

						end;
				end;	{ with }
		end;	{ DoTest }


{****************************************************}
{	DoCalcCntl	}
{****************************************************}
		procedure DoCalcCntl;	{new for System 7.0}
			var
				contrlRect: Rect;
		begin
			contrlRect := control^^.contrlRect;
			RectRgn(RgnHandle(param), contrlRect);
		end;	{ DoCalcCntl }


{****************************************************}
{	DoCalc	}
{****************************************************}
		procedure DoCalc;
			var
				contrlRect: Rect;
		begin
			if param < 0 then
				begin
					param := BAnd(param, $7FFFFFFF);	{ NOT 32-bit clean! }
				end
			else
				begin
					DoCalcCntl;
				end;
		end;	{ DoCalc }



	begin	{ CalendarCDEF }
		result := 0;

		GetPenState(savePen);
		PenNormal;

		if control^^.contrlValue = know then
			begin
				GetDateTime(secs);
				control^^.contrlValue := secs div kdaysecs - 1;
			end
		else
			begin
				secs := control^^.contrlValue;
				secs := kdaysecs * secs;
			end;

		SetRect(tableRect, 0, 0, 7 * hBox, 6 * vBox);
		SetRect(yearRect, 1, -2 * vBox - 1, kxlin, -vBox - 1);
		SetRect(monthRect, kxlin + 1, -2 * vBox - 1, 7 * hBox, -vBox - 1);
		SetRect(nowRect, 0, -vBox - 1, 7 * hBox, -1);

		case message of
			initCntl: 
				DoInit;
			dispCntl: 
				DoDispose;
			drawCntl: 
				DoDraw;
			testCntl: 
				DoTest;
			calcCRgns: 
				DoCalc;
			calcCntlRgn: 
				DoCalcCntl;
			otherwise
		end;	{ case }

		SetPenState(savePen);

		if (result = inCalendar) then
			control^^.contrlValue := GetDays;

		CalendarCDEF := result;
	end;		{ CalendarCDEF }

end.	{ CDEF }