{****************************************************}
{}
{ 		Dialog				}
{}
{		Hansruedi Baer		}
{		Department of Geography	}
{		University of Zurich		}
{		Winterthurerstrasse 190	}
{		CH-8057   Z u r i c h 		}
{		e-mail: baer@gis.geogr.unizh.ch	}
{}
{****************************************************}

unit Dialog;

interface

	uses
		CDEFTest;

	const
		dialogID = 128;
		kAppleMenu = 128;
		kFileMenu = kAppleMenu + 1;
		kEditMenu = kFileMenu + 1;
		atTheEnd = 0;
		kAboutItem = 1;
		kAbout = 256;
		kOpen = 1;
		kClose = kOpen + 1;
		kQuit = kClose + 1;
		kCutItem = 1;
		kCopyItem = kCutItem + 1;
		kPasteItem = kCopyItem + 1;
		kClearItem = kPasteItem + 1;
		SuspendResumeEvt = $01;		{ Suspend/Resume event code	}
		ResumeEvtMask = $1;			{ Supend or Resume selector	}


	procedure Run;


implementation


	procedure OpenDlg (dialogID: integer);
	forward;

	procedure CloseDlg;
	forward;

	var
		theDialog: DialogPtr;
		gStop: boolean;

{****************************************************}
{	SetUpMenus	}
{}
{		Install the menus.	}
{****************************************************}
	procedure SetUpMenus;
		var
			theMenu: MenuHandle;
	begin
		theMenu := GetMenu(kAppleMenu);
		InsertMenu(theMenu, atTheEnd);
		AddResMenu(theMenu, 'DRVR');
		theMenu := GetMenu(kFileMenu);
		InsertMenu(theMenu, atTheEnd);
		theMenu := GetMenu(kEditMenu);
		InsertMenu(theMenu, atTheEnd);
		DrawMenuBar;
		theDialog := nil;
		gStop := FALSE;
	end;


{****************************************************}
{	UpdateMenu	}
{}
{		Disable and enable the appropriate menu items.	}
{****************************************************}
	procedure UpdateMenu;
		var
			theMenu: MenuHandle;
			i: integer;

	begin

		theMenu := GetMenu(kFileMenu);
		for i := kOpen to kClose do
			DisableItem(theMenu, i);
		if theDialog = nil then
			EnableItem(theMenu, kOpen)
		else
			EnableItem(theMenu, kClose);

		theMenu := GetMenu(kEditMenu);
		for i := kCutItem to kClearItem do
			DisableItem(theMenu, i);
		if theDialog <> nil then
			EnableItem(theMenu, kCopyItem);

	end;


{****************************************************}
{	DoMenu	}
{}
{		Process menu commands.	}
{****************************************************}
	procedure DoMenu (packedMenu: Longint);
		var
			accName: Str255;
			accNumber: integer;
			dItem: integer;
			menu, item: integer;
	begin
		menu := HiWord(packedMenu);
		item := LoWord(packedMenu);
		case menu of

			kAppleMenu: 
				begin
					if (item = kAboutItem) then
						begin
							dItem := Alert(kAbout, nil);
						end
					else
						begin
							GetItem(GetMHandle(kAppleMenu), item, accName);
							accNumber := OpenDeskAcc(accName);
						end;
				end;

			kFileMenu: 
				begin
					case item of
						kOpen: 
							begin
								OpenDlg(dialogID);
							end;
						kClose: 
							begin
								CloseDlg;
							end;
						kQuit: 
							begin
								gStop := TRUE;
							end;
					end;
				end;

			kEditMenu: 
				begin
					case item of
						kCopyItem: 
							CopyControl(theDialog);
					end;
				end;

			otherwise
		end;
		HiliteMenu(0);
	end;


{****************************************************}
{	DoDialogEvent	}
{}
{		Handle dialog events	}
{****************************************************}
	procedure DoDialogEvent (theEvent: EventRecord);
		var
			whichDialog: DialogPtr;
			whichControl: ControlHandle;
			localPt: Point;
			itemHit, controlPart: integer;
	begin
		if IsDialogEvent(theEvent) then
			begin
				if DialogSelect(theEvent, whichDialog, itemHit) then
					begin
						localPt := theEvent.where;
						GlobalToLocal(localPt);
						controlPart := FindControl(localPt, whichDialog, whichControl);
						if whichControl <> nil then
							begin
								if (controlPart >= inUpButton) & (controlPart <= inPageDown) then
									controlPart := TrackControl(whichControl, localPt, nil);
							end;
						DoDialogItem(whichDialog, itemHit);
					end;
			end;
	end;


{****************************************************}
{	DoKeyDown	}
{}
{		Handle key down events	}
{****************************************************}
	procedure DoKeyDown (message: longint; modifiers: integer);
		var
			charCode: char;

	begin
		charCode := char(BitAnd(message, charCodeMask));
		if (BitAnd(modifiers, cmdKey) = cmdKey) then
			begin
				UpdateMenu;
				DoMenu(MenuKey(charCode));
			end;
	end;


{****************************************************}
{	DoMouseDown	}
{}
{		Handle mouse down events.	}
{****************************************************}
	procedure DoMouseDown (where: Point);
		var
			whichWindow: WindowPtr;
			bounds: Rect;
			windowPart: integer;

	begin
		windowPart := FindWindow(where, whichWindow);
		case windowPart of

			inGoAway: 
				if TrackGoAway(whichWindow, where) then
					CloseDlg;

			inDrag: 
				begin
					bounds := screenBits.bounds;
					InsetRect(bounds, 4, 4);
					bounds.top := bounds.top + 20;
					DragWindow(whichWindow, where, bounds);
				end;

			inMenuBar: 
				begin
					UpdateMenu;
					DoMenu(MenuSelect(where));
				end;

			otherwise

		end;
	end;


{****************************************************}
{	OpenDlg	}
{}
{		Open the dialog.	}
{****************************************************}
	procedure OpenDlg (dialogID: integer);
	begin
		theDialog := GetNewDialog(dialogID, nil, POINTER(-1));
		SetPort(theDialog);
		InitCursor;
		InitTest(theDialog);
	end;


{****************************************************}
{	CloseDlg	}
{}
{		Close the dialog.	}
{****************************************************}
	procedure CloseDlg;
	begin
		DisposDialog(theDialog);
		theDialog := nil;
	end;


{****************************************************}
{	Run	}
{}
{		Event loop.	}
{****************************************************}
	procedure Run;
		var
			theEvent: EventRecord;
			ignore: boolean;
	begin
		SetupMenus;
		OpenDlg(dialogID);

		repeat
			ignore := WaitNextEvent(everyEvent, theEvent, 60, nil);
			DoDialogEvent(theEvent);
			begin
				with theEvent do
					begin
						case theEvent.what of		{ handle mouse down events }
							mouseDown: 
								DoMouseDown(where);
							keyDown: 
								DoKeyDown(message, modifiers);
							activateEvt: 
								Doactivate(WindowPtr(message), odd(modifiers));
							app4Evt: 
								if BAND(BSR(message, 24), $FF) = SuspendResumeEvt then
									DoActivate(theDialog, BAND(theEvent.message, ResumeEvtMask) <> 0);
							otherwise
						end;
					end;
			end;
		until gStop;
	end;


end.